/*
 * alarm-properties.js
 *
 * Copyright (c) 2005 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Handles the UI for setting an alarm.
 */

const SOUND_FILES = "*.wav";

// The maximum number of characters before overflowing to "some" for alarm names
const MAX_NAME_CHARS = 20;

var rseRaces = ['elvaan.male', 'elvaan.female',
	'tarutaru.male', 'tarutaru.female',
	'mithra', 'galka', 'hume.male', 'hume.female' ];

var vanadielTimer;
var alarmManager;
var vanadielOffset = 0;
var edittedAlarm = null;

var selectDaysText;
var selectPhasesText;

function init() {
    /*sizeToContent();
    // remove the width constraints on the description
    var desc = document.getElementById("advanced-description");
    desc.setAttribute("style", "width: " + desc.parentNode.innerWidth + "px");
    desc.removeAttribute("width");*/
    sizeToContent();
    var x = Math.floor((window.opener.outerWidth - window.outerWidth) / 2 + window.opener.screenX);
    var y = Math.floor((window.opener.outerHeight - window.outerHeight) / 2 + window.opener.screenY);
    window.moveTo(x, y);
    vanadielTimer = Components.classes["@xenoveritas.org/vanadiel-timer;1"]
			 .getService(Components.interfaces.nsIVanadielTimer);
    vanadielOffset = vanadielTimer.vanadielOffset;
    // grab the alarm manager
    alarmManager = Components.classes["@xenoveritas.org/vanadiel-alarm-manager;1"].
               getService(Components.interfaces.nsIVanadielAlarmManager);
    var label = document.getElementById("alarm.label");
    // Due to ... bugs ... windows.arguments may be null
    var aID = null;
    if (window.arguments != null)
	aID = window.arguments.length >= 1 ? window.arguments[0] : null;
    if (aID == null || aID == "new") {
	document.getElementById("vanadielclock-set-alarm-window")
	    .setAttribute("title", document.getElementById("alarm-properties-strings").getString("window.title.create"));
    } else {
	// try and load an existing alarm from the alarm manager
	edittedAlarm = alarmManager.getAlarmById(aID);
	label.value = edittedAlarm.name;
	document.getElementById("alarm.description").value = edittedAlarm.description;
	if (edittedAlarm.sound == "none") {
	    document.getElementById("alarm-sound-group").selectedIndex = 0;
	} else if (edittedAlarm.sound == "beep") {
	    document.getElementById("alarm-sound-group").selectedIndex = 1;
	} else {
	    document.getElementById("alarm-sound-group").selectedIndex = 2;
	    document.getElementById("alarm-sound-custom-box").value = edittedAlarm.sound;
	}
	// Always default to "advanced" so something gets displayed even if
	// everything borks up
	var typeGroup = document.getElementById("type-group");
	typeGroup.selectedItem = document.getElementById("type-advance-radio");
	var type = edittedAlarm.type;
	var i = type.indexOf(" ");
	var pattern = edittedAlarm.pattern;
	if (i >= 0) {
	    pattern = type.substring(i + 1);
	    type = type.substring(0, i);
	}
	dump("Resurrecting alarm, [" + type + ":" + pattern + "]\n");
	// set up the existing data based on the alarm values
	if (type == "time") {
	    // figure out the day it sounds on
	    var toks = pattern.split(" ");
	    if (toks.length >= 4) {
		var days = toks[0].split(",");
		for (var i in days) {
		    var check = document.getElementById("day.check." + days[i]);
		    if (check != null)
			check.checked = true;
		}
		var phases = toks[3].split(",");
		for (var i in phases) {
		    var check = document.getElementById("moon.check." + phases[i]);
		    if (check != null)
			check.checked = true;
		}
		document.getElementById("time-box").value = toks[1] + ":" + (parseInt(toks[2]) < 10 ? "0" : "") + toks[2];
	    }
	    typeGroup.selectedItem = document.getElementById("type-time-radio");
	    updateDays();
	    updatePhases();
	} else if (type == "rse") {
	    var menulist = document.getElementById("rse-dropdown");
	    // this is easy, find the matching pattern
	    for (i = 0; i < menulist.menupopup.childNodes.length; i++) {
		if (menulist.menupopup.childNodes.item(i).getAttribute("icon") == pattern) {
		    // select this
		    menulist.selectedIndex = i;
		    break;
		}
	    }
	    typeGroup.selectedItem = document.getElementById("type-rse-radio");
	} else if (type == "guild") {
	    var menulist = document.getElementById("guild-dropdown");
	    i = pattern.indexOf(" ");
	    var open = true;
	    if (i >= 0) {
		open = pattern.substring(i+1) != "close";
		pattern = pattern.substring(0, i);
	    }
	    document.getElementById("guild-radio-group").selectedIndex = open ? 0 : 1;
	    // this is easy, find the matching pattern
	    for (i = 0; i < menulist.menupopup.childNodes.length; i++) {
		if (menulist.menupopup.childNodes.item(i).getAttribute("guild") == pattern) {
		    // select this
		    menulist.selectedIndex = i;
		    break;
		}
	    }
	    typeGroup.selectedItem = document.getElementById("type-guild-radio");
	}
	// synth the defaults, and if they match, kill 'em
	var alarm = createAlarm();
	if (label.value == alarm.name) {
	    label.value = "";
	}
	label = document.getElementById("alarm.description");
	if (label.value == alarm.description) {
	    label.value = "";
	}
	document.getElementById("advanced-textbox").value = alarm.pattern;
    }
    checkSoundBefore();
    checkAlarmSound();
    updateOptions();
}

function updateOptions() {
    updateDescription();
    updateTimeOptions();
    document.getElementById("rse-dropdown").disabled = !document.getElementById("type-rse-radio").selected;
    var disable = !document.getElementById("type-guild-radio").selected;
    document.getElementById("guild-dropdown").disabled = disable;
    document.getElementById("guild-radio-open").disabled = disable;
    document.getElementById("guild-radio-close").disabled = disable;
    document.getElementById("advanced-textbox").disabled = !document.getElementById("type-advance-radio").selected;
}

function updateDescription() {
    try {
	var alarm = createAlarm();
    } catch (ex) {
	dumpobject(ex);
    }
    document.getElementById("alarm.label").defaultValue = alarm.name;
    document.getElementById("alarm.description").defaultValue = alarm.description;
}

function checkSoundBefore() {
    document.getElementById("minutes.before").disabled = ! document.getElementById("sound-before-event").checked;
}

function checkAlarmSound() {
    var disabled = !document.getElementById("alarm-sound-custom").selected;
    document.getElementById("alarm-sound-custom-box").disabled = disabled;
    document.getElementById("alarm-sound-custom-browse").disabled = disabled;
    // there's really no point to having "test" enabled if "none" is chosen
    // Actually, there's little point in having it enabled if "beep" is chosen,
    // but I suppose people might want to know what "system default" is
    document.getElementById("alarm-sound-custom-test").disabled = document.getElementById("alarm-sound-none").selected;
}

function browseSound() {
    var strings = document.getElementById("alarm-properties-strings");
    var nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
	.createInstance(nsIFilePicker);
    fp.appendFilter(strings.getString("sound.files") + " (" + SOUND_FILES + ")", SOUND_FILES);
    fp.init(window, strings.getString("sound.browse.title"), nsIFilePicker.modeOpen);
    var res = fp.show();
    if (res == nsIFilePicker.returnOK) {
	var soundFile = fp.file;
	document.getElementById("alarm-sound-custom-box").value = soundFile.path;
    }
}

function testSound() {
    // see what type of sound we're using
    if (document.getElementById("alarm-sound-none").selected) {
	// that was mighty pointless
	return;
    }
    var soundService = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);
    soundService.init();
    if (document.getElementById("alarm-sound-beep").selected) {
	soundService.beep();
    } else {
	soundService.playSystemSound(document.getElementById("alarm-sound-custom-box").value);
    }
}

/**
 * Dumb little interpolator.
 */
function quickyInterpolate(str, key1, val1, key2, val2, key3, val3) {
    var re = new RegExp("\\{" + key1 + "\\}", "g");
    str = str.replace(re, val1);
    if (key2 != null) {
	re = new RegExp("\\{" + key2 + "\\}", "g");
	str = str.replace(re, val2);
    }
    if (key3 != null) {
	re = new RegExp("\\{" + key3 + "\\}", "g");
	str = str.replace(re, val3);
    }
    return str;
}

function trimQuotes(str) {
    if (str.length > 0) {
	if (str.substring(0,1) == '"') {
	    str = str.substring(1);
	}
    }
    if (str.length > 0) {
	if (str.substring(str.length - 1, str.length) == '"') {
	    str = str.substring(0, str.length - 1);
	}
    }
    return str;
}

/**
 * Creates a skeleton object for a given alarm.  This is used to "test"
 * the alarm using the "Test Alarm" button, or at least the secret keystroke
 * Accel-T
 */
function createAlarm() {
    var alarm = new Object();
    alarm.name = "unknown";
    alarm.description = "unknown";
    alarm.sound = "none";
    if (document.getElementById("alarm-sound-beep").selected) {
	alarm.sound = "beep";
    } else if (document.getElementById("alarm-sound-custom").selected) {
	alarm.sound = document.getElementById("alarm-sound-custom-box").value;
    } else {
	alarm.sound = "none";
    }
    var vanadielTime = vanadielTimer.time;
    var type = document.getElementById("type-group").selectedItem.getAttribute("id");
    var strings = document.getElementById("alarm-properties-strings");
    var listJoin = trimQuotes(strings.getString("list.join"));
    var listJoinTwo = trimQuotes(strings.getString("list.join.two"));
    var listJoinLast = trimQuotes(strings.getString("list.join.last"));
    alarm.smallIcon = "chrome://vanadielclock/skin/elements/fire";
    alarm.icon = alarm.smallIcon + "32.png";
    alarm.smallIcon = alarm.smallIcon + ".png";
    alarm.name = strings.getString("borked");
    alarm.description = strings.getString("borked.desc");
    alarm.pattern = "0 0 0 0 0";
    alarm.type = "time " + alarm.pattern;
    if (type == "type-time-radio") {
	// Create a new alarm based on the current time.
	var dayLabel = null;
	var daysLabel = null;
	var days = null;
	var dayGroup = document.getElementById("days-group");
	var len = dayGroup.childNodes.length;
	var allDays = true;
	for (var i = 0; i < len; i++) {
	    var item = dayGroup.childNodes.item(i);
	    if (item.getAttribute("selected") == "true") {
		var day = item.getAttribute("day");
		if (day != null) {
		    if (days == null) {
			days = day;
		    } else {
			days = days + "," + day;
		    }
		    day = document.getElementById("day.check." + day).getAttribute("label");
		    if (dayLabel == null) {
			dayLabel = day;
		    } else {
			daysLabel = dayLabel + (daysLabel == null ? listJoinTwo : listJoinLast) + day;
			dayLabel = dayLabel + listJoin + day;
		    }
		}
	    } else {
		allDays = false;
	    }
	}
	if (days == null) {
	    allDays = true;
	}
	if (allDays) {
	    days = "*";
	    daysLabel = "-all-";
	}
	var phaseLabel = null;
	var phasesLabel = null;
	var phases = null;
	var phaseGroup = document.getElementById("phases-group");
	len = phaseGroup.childNodes.length;
	var allPhases = true;
	for (var i = 0; i < len; i++) {
	    var item = phaseGroup.childNodes.item(i);
	    if (item.getAttribute("selected") == "true") {
		var phase = item.getAttribute("id");
		if (phase != null) {
		    phase = phase.substring(phase.lastIndexOf('.')+1);
		    if (phases == null) {
			phases = phase;
		    } else {
			phases = phases + "," + phase;
		    }
		    phase = document.getElementById("moon.check." + phase).getAttribute("label");
		    if (phaseLabel == null) {
			phaseLabel = phase;
		    } else {
			phasesLabel = phaseLabel + (phasesLabel == null ? listJoinTwo : listJoinLast) + phase;
			phaseLabel = phaseLabel + listJoin + phase;
		    }
		}
	    } else {
		allPhases = false;
	    }
	}
	if (phases == null) {
	    allPhases = true;
	}
	if (allPhases) {
	    phases = "*";
	    phasesLabel = "-all-";
	}
	var hour = document.getElementById("time-box").value;
	var minute = hour.indexOf(":");
	if (minute >= 0) {
	    minute = parseInt(hour.substring(minute+1), 10);
	    if (!(minute >= 0 && minute < 60))
		minute = 0;
	} else {
	    minute = 0;
	}
	hour = parseInt(hour, 10);
	if (!(hour >= 0 && hour < 24))
	    hour = 0;
	var hourLabel = hour + ":" + (minute < 10 ? "0" : "") + minute;
	alarm.smallIcon = "chrome://vanadielclock/skin/elements/fire";
	alarm.icon = alarm.smallIcon + "32.png";
	alarm.smallIcon = alarm.smallIcon + ".png";
	alarm.name = "borked";
	alarm.description = "borked.desc";
	var dayLabelPart = "";
	if (daysLabel == null) {
	    dayLabelPart = ".one";
	} else if (daysLabel.length > MAX_NAME_CHARS) {
	    dayLabelPart = ".some";
	}
	var phaseLabelPart = "";
	if (phasesLabel == null) {
	    phaseLabelPart = ".one";
	} else if (phasesLabel.length > MAX_NAME_CHARS) {
	    phaseLabelPart = ".some";
	}
	alarm.icon = "time:day";
	alarm.smallIcon = "time:day";
	if (allDays && allPhases) {
	    alarm.name = "time.all";
	} else if (allDays) {
	    alarm.name = "time.phase" + phaseLabelPart;
	    alarm.icon = "time:moon";
	    alarm.smallIcon = "time:moon";
	} else if (allPhases) {
	    alarm.name = "time.day" + dayLabelPart;
	} else {
	    alarm.name = "time.phase" + phaseLabelPart + ".day" + dayLabelPart;
	}
	alarm.description = quickyInterpolate(strings.getString(alarm.name + ".desc"), "time", hourLabel, "day", daysLabel == null ? dayLabel : daysLabel, "phase", phasesLabel == null ? phaseLabel : phasesLabel)
	alarm.name = quickyInterpolate(strings.getString(alarm.name), "time", hourLabel, "day", daysLabel == null ? dayLabel : daysLabel, "phase", phasesLabel == null ? phaseLabel : phasesLabel)
	alarm.pattern = days + " " + hour + " " + minute + " " + phases;
	alarm.type = "time " + alarm.pattern;
    } else if (type == "type-rse-radio") {
	var item = document.getElementById("rse-dropdown").selectedItem;
	alarm.name = quickyInterpolate(strings.getString("rse.name"), "race", item.label);
	alarm.description = quickyInterpolate(strings.getString("rse.desc"), "race", item.label);
	alarm.pattern = item.getAttribute("pattern");
	alarm.type = "rse " + item.getAttribute("icon");
	alarm.smallIcon = "chrome://vanadielclock/skin/icons/races/" + item.getAttribute("icon");
	alarm.icon = alarm.smallIcon + "32.png";
	alarm.smallIcon = alarm.smallIcon + ".png";
    } else if (type == "type-guild-radio") {
	var item = document.getElementById("guild-dropdown").selectedItem;
	var open = document.getElementById("guild-radio-open").selected;
	var statusLocal = document.getElementById("guild-radio-" + (open ? "open" : "close")).label;
	alarm.name = quickyInterpolate(strings.getString("guild.name"), "guild", item.label, "status", statusLocal);
	alarm.description = quickyInterpolate(strings.getString("guild.desc"), "guild", item.label, "status", statusLocal);
	alarm.pattern = item.getAttribute("pattern-day") + " " + item.getAttribute("pattern-" + (open ? "open" : "close")) + " 0";
	alarm.type = "guild " + item.getAttribute("guild") + " " + (open ? "open" : "close");
	alarm.smallIcon = "chrome://vanadielclock/skin/icons/guilds/" + item.getAttribute("guild");
	alarm.icon = alarm.smallIcon + "32.png";
	alarm.smallIcon = alarm.smallIcon + ".png";
    } else if (type == "type-advance-radio") {
	alarm.name = strings.getString("advanced.name");
	alarm.description = strings.getString("advanced.desc");
	alarm.pattern = document.getElementById("advanced-textbox").value;
	alarm.type = "advanced";
    } else {
	//alert("You managed to choose an alarm type (" + type + ") that isn't listed.  Have a cookie!  But no alarm will be set, sorry.");
	//return;
    }
    return alarm;
}

function testAlarm() {
    var alarm = createAlarm();
    var alertService = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
    alertService.showAlertNotification(alarm.icon, alarm.name, alarm.description, false, "don't care", null);
    testSound();
}

function acceptChanges() {
    // Create the alarm
    var jsAlarm = createAlarm();
    var xpcAlarm = edittedAlarm == null ?
    			Components.classes["@xenoveritas.org/vanadiel-alarm;1"].
				createInstance(Components.interfaces.nsIVanadielAlarm) :
			edittedAlarm;
    // set the properties
    xpcAlarm.name = document.getElementById("alarm.label").value;
    xpcAlarm.description = document.getElementById("alarm.description").value;
    xpcAlarm.icon = jsAlarm.icon;
    xpcAlarm.smallIcon = jsAlarm.smallIcon;
    xpcAlarm.sound = jsAlarm.sound;
    xpcAlarm.type = jsAlarm.type;
    if (document.getElementById("sound-before-event").checked) {
	var i = parseInt(document.getElementById("minutes.before"));
	if (i > 0) {
	    i *= 60 * 25;
	} else {
	    i = 1 * 60 * 25;
	}
	dump("setting alarm to go off at " + i + " before\n");
	xpcAlarm.preAlertTime = i;
    }
    if (document.getElementById("alarm-repeat").checked) {
	xpcAlarm.pattern = jsAlarm.pattern;
	xpcAlarm.updateAlarm(VanadielClock_TimerService.time);
    } else {
	xpcAlarm.pattern = "";
	xpcAlarm.setAlarm(VanadielClock_TimerService.time, jsAlarm.pattern);
    }
    if (edittedAlarm == null)
	alarmManager.addAlarm(xpcAlarm);
    else if (window.opener.buildAlarmTree) {
	window.opener.buildAlarmTree();
    }
}

function disableChildren(node, disable) {
    if (node.nodeType == document.ELEMENT_NODE) {
	node.setAttribute("disabled", disable);
	var children = node.childNodes;
	for (var i = 0; i < children.length; i++) {
	    disableChildren(children[i], disable);
	}
    }
}

/** Updates the widgets for setting an event based on the time */
function updateTimeOptions() {
    var disable = !document.getElementById("type-time-radio").selected;
    document.getElementById("days-check").disabled = disable;
    document.getElementById("days-group").setAttribute("disabled", disable ? "true" : "false");
    document.getElementById("phases-check").disabled = disable;
    document.getElementById("phases-group").setAttribute("disabled", disable ? "true" : "false");
    document.getElementById("time-check").disabled = disable;
    document.getElementById("time-box").disabled = disable;
    var group = document.getElementById("days-group");
    var len = group.childNodes.length;
    for (var i = 0; i < len; i++) {
	var item = group.childNodes.item(i);
	if (disable) {
	    item.setAttribute("disabled", "true");
	} else {
	    item.setAttribute("disabled", item.getAttribute("selected") == "true" ? "false" : "true");
	}
    }
    group = document.getElementById("phases-group");
    len = group.childNodes.length;
    for (var i = 0; i < len; i++) {
	var item = group.childNodes.item(i);
	if (disable) {
	    item.setAttribute("disabled", "true");
	} else {
	    item.setAttribute("disabled", item.getAttribute("selected") == "true" ? "false" : "true");
	}
    }
}

function checkAll(checkbox, popup) {
    var children = document.getElementById(popup).childNodes;
    var len = children.length;
    var checked = checkbox.checked;
    for (var i = 0; i < len; i++) {
	var id = children[i].id;
	if (id.length > 4 && id.substr(id.length-4) == ".all") {
	    // ignore this
	} else {
	    children[i].checked = checked;
	}
    }
}

function toggleDay(event) {
    // toggle the selected attribute
    var selected = event.target.getAttribute("selected") == "true";
    event.target.setAttribute("selected", selected ? "false" : "true");
    // copy over to disabled (sorta)
    event.target.setAttribute("disabled", selected ? "true" : "false");
    document.getElementById("day.check." + event.target.getAttribute("day")).checked = !selected;
    updateDescription();
}

function updateDay(event) {
    // find out the selected day
    var day = event.target.getAttribute("element");
    if (day == null || day == "") // ignore the "all" check
	return;
    var selected = event.target.checked;
    var icon = document.getElementById("day-" + day + "-icon");
    icon.setAttribute("selected", selected ? "true" : "false");
    // copy over to disabled (sorta)
    icon.setAttribute("disabled", selected ? "false" : "true");
    updateDescription();
}

// updates all days based on the checked days in the popup
function updateDays() {
    for (var i = 0; i <= 7; i++) {
	var check = document.getElementById("day.check." + i);
	var icon = document.getElementById("day-" + check.getAttribute("element") + "-icon");
	icon.setAttribute("selected", check.checked ? "true" : "false");
	icon.setAttribute("disabled", check.checked ? "false" : "true");
    }
    updateDescription();
}

function togglePhase(event) {
    // toggle the selected attribute
    var selected = event.target.getAttribute("selected") == "true";
    event.target.setAttribute("selected", selected ? "false" : "true");
    // copy over to disabled (sorta)
    event.target.setAttribute("disabled", selected ? "true" : "false");
    var num = event.target.getAttribute("id");
    num = num.substring(num.lastIndexOf('.')+1);
    document.getElementById("moon.check." + num).checked = !selected;
    updateDescription();
}

function updatePhase(event) {
    // find out the selected phase
    var phase = event.target.getAttribute("moon");
    if (phase == null || phase == "") // ignore the "all" check
	return;
    var selected = event.target.checked;
    var icon = document.getElementById("moon.icon." + phase);
    icon.setAttribute("selected", selected ? "true" : "false");
    // copy over to disabled (sorta)
    icon.setAttribute("disabled", selected ? "false" : "true");
    updateDescription();
}

// updates all days based on the checked days in the popup
function updatePhases() {
    for (var i = 0; i <= 11; i++) {
	var check = document.getElementById("moon.check." + i);
	var icon = document.getElementById("moon.icon." + i);
	icon.setAttribute("selected", check.checked ? "true" : "false");
	icon.setAttribute("disabled", check.checked ? "false" : "true");
    }
    updateDescription();
}

function dumpobject(obj) {
    dump("Details for [" + obj + "]\n");
    for (var k in obj) {
	dump("obj." + k + " = [" + obj[k] + "];\n");
    }
}

function pixToNum(str) {
    var v = parseFloat(str);
    return v == NaN ? 0 : v;
}

/**
 * Fixes a problem under Windows XP (and who knows what else) where the tooltip
 * size would be calculated correctly but show up wrong anyway.
 */
function unbreakTooltip(tooltip) {
    tooltip.popupBoxObject.sizeTo(tooltip.popupBoxObject.width, tooltip.popupBoxObject.height);
}
