// For those of you running ancient browsers: the install.js file!
// This allows you to install the extension into Mozilla.
// Joy.

const APP_DISPLAYNAME = "Vana'diel Clock";
const APP_NAME = "/Xenoveritas/VanadielClock";
const APP_AUTHOR = "Dan Potter";
const APP_VERSION = "0.6.7";
const APP_CHROME = "vanadielclock";
const JAR_NAME = "vanadielclock.jar";

var folder = getFolder("Profile", "chrome");

var installInApp     = File.exists(getFolder(getFolder("chrome"), JAR_NAME));
var installInProfile = File.exists(getFolder(folder, JAR_NAME));

initInstall(APP_DISPLAYNAME, APP_NAME, APP_VERSION);

// by default, we install for the given user only
// we can also install for all users

if (!installInApp && !installInProfile) {
    if (confirm("By default this extension will install into your individual profile.\r\nIf you would instead like to install it so that it is available to all Mozilla users, press Cancel.")) {
	installInProfile = true;
    } else {
	installInApp = true;
    }
}

var flag;

if (installInProfile) {
    flag = PROFILE_CHROME;
} else {
    flag = 0;
    folder = getFolder("chrome");
}

setPackageFolder(folder);

var error = addFile(APP_AUTHOR, APP_VERSION, 'chrome/' + JAR_NAME, folder, null);

var errMsg = "Unable to extract chrome.";

if (error == SUCCESS) {
    // Install our default prefs - I guess this is safe-ish, although we can't do this
    // for multiple users (growls at Mozilla) - plus the predefined "Defaults" apparently
    // doesn't work - it returned an empty string ><
    var prefFolder = getFolder("Program", "defaults/pref");
    error = addFile(APP_AUTHOR, APP_VERSION, 'defaults/preferences/' + APP_CHROME + '.js', prefFolder, null);
    errMsg = "Unable to extract the default preferences.";
}

if (error == SUCCESS) {
    // install our components
    var componentFolder = getFolder("Components");
    error = addFile("nsVanadielAlarm.js", APP_VERSION, 'components/nsVanadielAlarm.js', componentFolder, "");
    if (error == SUCCESS) {
	error = addFile("nsVanadielAlarm.xpt", APP_VERSION, 'components/nsVanadielAlarm.xpt', componentFolder, "");
    }
    if (error == SUCCESS) {
	error = addFile("nsVanadielTimer.js", APP_VERSION, 'components/nsVanadielTimer.js', componentFolder, "");
	if (error == SUCCESS) {
	    error = addFile("nsVanadielTimer.xpt", APP_VERSION, 'components/nsVanadielTimer.xpt', componentFolder, "");
	}
    }
    // attempt to delete the component registry
    File.remove(getFolder("Components", "compreg.dat"));
    File.remove(getFolder("Components", "xpti.dat"));
    errMsg = "Unable to install components.";
}

if (error == SUCCESS) {
    folder = getFolder(folder, JAR_NAME);
    registerChrome(Install.CONTENT | flag, folder, "content/" + APP_CHROME + "/");
    registerChrome(Install.LOCALE | flag, folder, "locale/en-US/" + APP_CHROME + "/");
    registerChrome(Install.SKIN | flag, folder, "skin/classic/" + APP_CHROME + "/");

    error = performInstall();
    if (error == SUCCESS) {
	alert(APP_DISPLAYNAME + " version " + APP_VERSION + " has been successfully installed into your browser.");
    } else if (error == 999) {
	// Error 999 appears to be "restart web browser to continue"
	alert(APP_DISPLAYNAME + " version " + APP_VERSION + " has been successfully installed into your browser.\r\n\r\nYou will need to restart your browser to use it.");
    } else {
	alert("Failed to install " + APP_DISPLAYNAME + " version " + APP_VERSION + ":\r\n" + error);
	cancelInstall(error);
    }
} else {
    alert("Failed to install " + APP_DISPLAYNAME + " version " + APP_VERSION + ".\r\nMake sure you have the proper permissions to install the JAR file into your chrome directory.\r\nThe reported error was: " + error + "\r\n" + errMsg);
    cancelInstall(error);
}
